(a) #Gene shaving#: there are one main function "shave" with three
subroutines: "pcs.new", "lmfit" and "showclust". For example, suppose that
"data.tsv" is a data set in the working directory in Unix. To get 4
clusters with 15 permutation, say, of gene shaving for "data.tsv" in
Splus, using

> shave(read.table("data.tsv",header=T),4,15)

Outputs: (1) gap plots; (2) variance plots; (3) cluster plots; (4) labels
of gene names in each cluster.



(b) #Supervised/partial gene shaving#: there are one main function
"super.part" with four subroutines: "super.id", "pcs.sup", "lmfit" and
"showclust". For example, suppose that "data.tsv" is a dataset in the
working directory in Unix, and "data.clf" is a indicator file to classify
cells/samples into 3 groups (say).

(i) to get 2 clusters with 15 permutation of supervised gene shaving for
"data.tsv" in Splus, using

> super.part(read.table("data.tsv",header=T),scan("data.clf"),1,2,15)

(ii) to get 2 clusters with 15 permutation of partial supervised gene
shaving with lamda=0.5 for "data.tsv" in Splus, using

> super.part(read.table("data.tsv",header=T),scan("data.clf"),0.5,2,15)

Outputs are (1) gap plots; (2) variance plots; (3) cluster plots; (4)
labels of gene names in each cluster.


####################################################################################

> shave
function(A, NumClusts, NumPerm)
{
        # this program will show all plots on one sheet...
        alpha <- 0.9
        N <- nrow(A)
        P <- ncol(A)
        A <- as.matrix(A)
        par(mfrow = c(1, 1), cex = 0.3)
        showclust(A, 1)
        par(mfrow = c(4, 3), cex = 0.3)
        ### A is row-centered:
        A <- sweep(A, 1, apply(A, 1, mean))
        CurrentMatrix <- A
        ##then, compute how many shaving times are needed if N>10, after
        ##which we shave just one gene at a time:
        account <- 0
        SizeNested <- N
        ##save some space:
        N.shave <- max(40,  - log(N)/log(alpha))
        size.after.shaving <- numeric(N.shave)
        while(SizeNested > 2) {
                SizeNested <- floor(SizeNested * alpha)
                account <- account + 1
                size.after.shaving[account] <- SizeNested
        }
        ##thus, we need "account" times of shaving to reach 2 genes
        Dstar <- matrix(0, account, NumPerm)
        ## R-square will be saved in Dstar in each permutation
        clust <- 1
        while(clust < (NumClusts + 1)) {
                gapsize <- pcs.new(CurrentMatrix, account, alpha)
                ##R-square and between-variance:
                Dk <- gapsize$R2
                VB <- gapsize$vb
                ###permute CurrentMatrix, then repeat the above steps: 
                perm <- 0
                while(perm < NumPerm) {
                        perm <- perm + 1
                        Z <- t(apply(CurrentMatrix, 1, sample))
                        ## permute within each row
                        Dstar[, perm] <- pcs.new(Z, account, alpha)$R2
                }
                Dk.R <- apply(Dstar, 1, mean)
                ##Dk.R is mean of R-square of the permuted data
                Gap <- Dk - Dk.R
                if(clust == 5) {
                        Motif()
                        par(mfrow = c(4, 3))
                }
                x <- size.after.shaving[size.after.shaving != 0]
                plot(x, Gap, log = "x", xlab = "Cluster Size")
                subtitl <- paste("Cluster #", clust)
                plot(x, Dk, pch = 0, log = "x", xlab = "Cluster Size", ylab = 
                        "Variance", ylim = c(0, 1), col = 1)
                points(x, Dk.R, pch = 1, col = 2)
                key(text = list(c("Real Data", "Randomized Data")), type = c(
                        "p", "p"), col = c(1, 2), points = list(pch = c(0,
                        1)))
                title(main = subtitl)
                vartitle <- paste("Cluster #", clust)
                title(main = vartitle)
                ##now, esitmate the optimal cluster in which  the label of
                ##genes is denoted by xx:
                xx <- gapsize$IndGen[[order( - Gap)[1]]]
                print(xx)
                #alternatively, you might write it out:
                #write(xx, "GClabel", ncolumns = length(xx), append = T)
                #write(length(xx), "GClength", append = T)
                showclust(A[xx,  ], 2)
                ## then, show %variance,VB,VT,VW,...:
                variance <- round(Dk[Gap == max(Gap)], 3)
                vbetween <- round(VB[Gap == max(Gap)], 2)
                vtotal <- round(vbetween/variance, 2)
                vbvw <- round(Dk[Gap == max(Gap)]/(1 - Dk[Gap == max(Gap)]),
                        2)
                subtitl <- paste("%variance=", variance * 100, "VB/VW=", vbvw,
                        "VB=", vbetween, "VT=", vtotal)
                title(main = subtitl)
                ## start to search for another cluster:
                clust <- clust + 1
                if(clust > NumClusts) {
                        break
                }
                xbar <- gapsize$z.bar[order( - Gap)[1],  ]
                CurrentMatrix <- cbind(CurrentMatrix, matrix(rep(xbar, N),
                        N, P, byrow = T))
                ##orth. CurrentMatrix:
                CurrentMatrix <- t(apply(CurrentMatrix, 1, lmfit))
                dimnames(CurrentMatrix) <- dimnames(A)
                CurrentMatrix <- sweep(CurrentMatrix, 1, apply(CurrentMatrix,
                        1, mean))
        }
        return()
}
> pcs.new
function(z, acc, alpha)
{
        R2 <- vb <- rep(0, acc)
        IndGen <- list(acc)
        z.bar <- matrix(0, acc, dim(z)[[2]])
        i <- 0
        repeat {
                i <- i + 1
                #pca <- as.vector(abs(svd(z)$u[, 1]))
                pca <- as.vector(svd(z)$u[, 1])
                genesign <- ifelse(pca > 0, 1, -1)
                z1 <- z * genesign
                pca <- abs(pca)
                if(length(pca) > 2) {
                        k <- length(pca) - floor(alpha * length(pca))
                }
                if(length(pca) == 2)
                        break
                z <- z[ - (order(pca)[1:k]),  ]
                z.sign <- z1[ - (order(pca)[1:k]),  ]
                z.bar[i,  ] <- apply(z.sign, 2, mean)
                d <- dim(z)
                IndGen[[i]] <- dimnames(z)[[1]]
                vb[i] <- (var(apply(z.sign, 2, mean), SumSquares = T))/d[[2]]
                vw <- mean(colVars(z.sign, SumSquares = T))/d[[1]]
                bw <- vb[i]/vw
                R2[i] <- bw/(1 + bw)
        }
        return(R2, vb, IndGen, z.bar)
}
> lmfit
function(x)
{
        fit <- lm(as.vector(x[1:(length(x)/2)]) ~ x[(length(x)/2 + 1):(length(
                x))])
        return(fit$res)
}
> showclust
function(z, flag)
{
        if(flag == 1) {
                #graph <- image(t(z), axes = F)
                x <- c(1:dim(z)[1])
                y <- c(1:dim(z)[2])
                graph <- image(x, y, z, axes = F)
                axis(1, at = x, labels = dimnames(z)[[1]], srt = 90, adj = 0.8,
                        col = 1, cex = 0.45)
                axis(2, at = y, labels = dimnames(z)[[2]], adj = 0.7, col = 1,
                        cex = 0.45)
        }
        else {
                y <- c(1:dim(z)[1])
                x <- c(1:dim(z)[2])
                tmp <- rbind(z, apply(z, 2, mean))
                tmp <- tmp[1:(dim(z)[1] + 1), order(tmp[(dim(z)[1] + 1),  ])]
                z <- tmp[ - (1 + dim(z)[1]),  ]
                graph <- image(x, y, t(z), axes = F)
                axis(1, at = x, labels = dimnames(t(z))[[1]], srt = 90, adj = 
                        0.9, col = 1, cex = 0.45)
                axis(2, at = y, labels = dimnames(t(z))[[2]], adj = 0.6, col = 
                        1, cex = 0.45)
        }
        print(graph)
}
> 

#####################################################################
> super.part
function(A, clf, lam, NumClusts, NumPerm)
{
        #######################################################################
        # clf is a vector which indicates the cell classification, for example, 
        # if the first 4 cells and last 3 cells (7 cells in total,say) are two 
        # different groups, then clf is (1,1,1,1,2,2,2). 
        # lam is supervised parameter and takes values in [0,1]. 
        # lam =1 if supervised; lam <1 if partial supervised.
        # An example of running this program for gulob data in Splus will be:  
        #
        #>super.part(read.table("data.tsv",header=T),scan("data.clf"),1,2,8)
        ###################################################################### 
        alpha <- 0.9
        N <- nrow(A)
        P <- ncol(A)
        A <- as.matrix(A)
        par(mfrow = c(1, 1), cex = 0.4)
        showclust(A, 1)
        par(mfrow = c(3, 3), cex = 0.3)
        A <- sweep(A, 1, apply(A, 1, mean))
        ### A is row-centered
        CurrentMatrix <- rbind(A, clf)
        CurrentMatrix <- CurrentMatrix[, order(as.numeric(CurrentMatrix[(N +
                1),  ]))]
        Q <- super.id(CurrentMatrix[(N + 1),  ], lam)
        ## a function 'super.id' produces a matrix Q such that AQ is 
        ## the working matrix (not A) in supervised shaving
        A <- CurrentMatrix <- CurrentMatrix[(1:N),  ]
        ##Compute how many shaving times are needed if N > 10, after which we
        ##shave just one gene at a time:
        account <- 0
        SizeNested <- N
        N.shave <- max(40,  - log(N)/log(alpha))
        size.after.shaving <- numeric(N.shave)
        while(SizeNested > 2) {
                SizeNested <- floor(SizeNested * alpha)
                account <- account + 1
                size.after.shaving[account] <- SizeNested
        }
        ###That is, we need "account" times of shaving to reach 2 genes
        Dstar <- matrix(0, account, NumPerm)
        clust <- 1
        while(clust < (NumClusts + 1)) {
                gapsize <- pcs.sup(CurrentMatrix, Q, account, alpha)
                Dk <- gapsize$R2
                VB <- gapsize$vb
                # R^2
                ###permute CurrentMatrix, then repeat the above steps: 
                perm <- 0
                while(perm < NumPerm) {
                        perm <- perm + 1
                        Z <- t(apply(CurrentMatrix, 1, sample))
                        ## permute within each row
                        Dstar[, perm] <- pcs.sup(Z, Q, account, alpha)$R2
                }
                Dk.R <- apply(Dstar, 1, mean)
                Gap <- Dk - Dk.R
                if(clust == 5) {
                        Motif()
                        par(mfrow = c(4, 3))
                }
                x <- size.after.shaving[size.after.shaving != 0]
                plot(x, Gap, log = "x", xlab = "Cluster Size")
                subtitl <- paste("Cluster #", clust)
                #title(main = subtitl)
                plot(x, Dk, pch = 0, log = "x", xlab = "Cluster Size", ylab = 
                        "Variance", ylim = c(0, 1), col = 1)
                points(x, Dk.R, pch = 1, col = 2)
                key(text = list(c("Real Data", "Randomized Data")), type = c(
                        "p", "p"), col = c(1, 2), points = list(pch = c(0,
                        1)))
                title(main = subtitl)
                vartitle <- paste("Cluster #", clust)
                title(main = vartitle)
                ##esitmate the optimal cluster in which  the label of genes is
                ##denoted by xx:
                xx <- gapsize$IndGen[[order( - Gap)[1]]]
                print(xx)
                #write(xx, "GClabel", ncolumns = length(xx), append = T)
                #write(length(xx), "GClength", append = T)
                showclust(A[xx,  ], 2)
                variance <- round(Dk[Gap == max(Gap)], 3)
                vbetween <- round(VB[Gap == max(Gap)], 2)
                vtotal <- round(vbetween/variance, 2)
                vbvw <- round(Dk[Gap == max(Gap)]/(1 - Dk[Gap == max(Gap)]),
                        2)
                subtitl <- paste("%variance=", variance * 100, "VB/VW=", vbvw,
                        "VB=", vbetween, "VT=", vtotal)
                title(main = subtitl)
                clust <- clust + 1
                if(clust > NumClusts) {
                        break
                }
                xbar <- gapsize$z.bar[order( - Gap)[1],  ]
                CurrentMatrix <- cbind(CurrentMatrix, matrix(rep(xbar, N),
                        N, P, byrow = T))
                CurrentMatrix <- t(apply(CurrentMatrix, 1, lmfit))
                ##orth
                dimnames(CurrentMatrix) <- dimnames(A)
                #showclust(CurrentMatrix, 1)
                CurrentMatrix <- sweep(CurrentMatrix, 1, apply(CurrentMatrix,
                        1, mean))
        }
        return()
}
> super.id
function(y, lam)
{
        k <- max(y)
        x <- rep(0, k)
        for(i in 1:k) {
                x[i] <- length(y[y == i])
        }
        d <- 0
        Q <- matrix(rep(0, sum(x) * k), sum(x), k)
        Q[, 1] <- c(rep(1, x[1]), rep(0, (sum(x) - x[1])))/x[1]
        for(i in 2:k) {
                d <- d + x[i - 1]
                tmp <- (d + 1):(d + x[i])
                Q[tmp, i] <- rep(1, length(tmp))/x[i]
        }
        Q <- lam * Q + (1 - lam) * ifelse(Q != 0, 1, 0)
        Q <- Q^0.5
        return(Q)
}
> pcs.sup
function(z, Q, acc, alpha)
{
        R2 <- vb <- rep(0, acc)
        IndGen <- list(acc)
        z.bar <- matrix(0, acc, dim(z)[[2]])
        i <- 0
        repeat {
                i <- i + 1
                pca <- as.vector(svd(z %*% Q)$u[, 1])
                #tmp <- eigen(t(Q) %*% t(z) %*% z %*% Q)
                #pca <- as.vector(z %*% Q %*% tmp$vectors %*% diag(1/tmp$values^
                #       0.5)[, 1])
                genesign <- ifelse(pca > 0, 1, -1)
                z1 <- z * genesign
                pca <- abs(pca)
                if(length(pca) > 2) {
                        k <- length(pca) - floor(alpha * length(pca))
                }
                if(length(pca) == 2)
                        break
                z <- z[ - (order(pca)[1:k]),  ]
                z.sign <- z1[ - (order(pca)[1:k]),  ]
                z.bar[i,  ] <- apply(z.sign, 2, mean)
                d <- dim(z)
                IndGen[[i]] <- dimnames(z)[[1]]
                vb[i] <- (var(apply(z.sign, 2, mean), SumSquares = T))/d[[2]]
                vw <- mean(colVars(z.sign, SumSquares = T))/d[[1]]
                bw <- vb[i]/vw
                R2[i] <- bw/(1 + bw)
        }
        return(R2, vb, IndGen, z.bar)
}
#######################################################################
# Two simulation data sets can be generated by the "gendat" and "gendat2"
# functions in Splus
#######################################################################
> gendat
function(N, p, b)
{
        ############################################################
        # this is a simulation matrix with N rows/genes and p samples, 
        # (No overlap in genes). b is the signal (b=3, for example)
        ###########################################################
        set.seed(1)
        a <- matrix(rep(b, (p * p)/4), p/2, p/2)
        a <- cbind( - a, a)
        b1 <- matrix(b, p/2, p)
        b2 <- matrix(diag(rep(c(1, -1), p/2)), p, p)
        b1 <- b1 %*% b2
        z <- rbind(a, b1, matrix(0, (N - p), p))
        ##noise generated from N(0,1):
        r <- rnorm(N * p)
        z <- z - matrix(r, N, p)
        dimnames(z) <- list(paste("G", 1:N, sep = ""), paste("S", 1:p, sep
                            = ""))
        return(z)
}


#######################################################
> gendat2
function(n, b,overlap)
{
        #####################################################
        ## this is a simulation matrix. Be sure that overlap < n, 
        ## b is the signal (b=3, for example). For example,
        ## 
        ## > gendat2(20,3,5)       
        #######################################################
        Q <- matrix(0, (3 * n - 2 * overlap), 3)
        Q[, 1] <- c(rep(1, n), rep(0, (dim(Q)[1] - n)))
        Q[, 2] <- c(rep(0, (n - overlap)), rep(1, n), 
                       rep(0, (n - overlap)))
        Q[, 3] <- c(rep(0, (2 * n - 2 * overlap)), rep(1, n))
        Q <- rep(as.data.frame(Q), 5)
        Q <- as.matrix(b * Q)
        Q <- Q[(1:dim(Q)[1]), order( - Q[1,  ],  - Q[n,  ])]
        #noise generated from N(0,1): 
        r <- rnorm(length(Q))
        Q <- Q + matrix(r, dim(Q)[1], dim(Q)[2])
        dimnames(Q) <- list(paste("Gene", 1:dim(Q)[1]), paste("S",
                                 1:dim(Q)[2]))
        return(Q)
}

